package be.iminds.ilabt.jfed.restapi.json;

import be.iminds.ilabt.jfed.restapi.json.util.JsonGeniUrnDeserializer;
import be.iminds.ilabt.jfed.restapi.json.util.JsonGeniUrnSerializer;
import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.util.jsonld.JsonLdObjectsMetaData;
import be.iminds.ilabt.util.jsonld.impl.PrimaryIdObject;
import com.fasterxml.jackson.annotation.*;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonIgnoreProperties("@context")
@JsonLdObjectsMetaData.JsonLdObjectInfo(pathName = "user", builderClass = UserBuilder.class,
uriType = JsonLdObjectsMetaData.UriType.ID_BASED, idType = JsonLdObjectsMetaData.IdType.PRIMARY)
public class User extends PrimaryIdObject<String> {
    private static final Logger LOG = LoggerFactory.getLogger(User.class);

    private final GeniUrn urn;
    private final String authorityTld;
    private final URL authorityServer;

    private final List<Slice> slices;
    private final List<Project> projects;

    @JsonCreator
    User(@JsonProperty("id") String id,
         @JsonProperty("urn") GeniUrn urn,
         @JsonProperty("authorityTld") String authorityTld,
         @JsonProperty("authorityServer") URL authorityServer,
         @JsonProperty("slices") List<Slice>  slices,
         @JsonProperty("projects") List<Project> projects,
         @JsonProperty("@id") URI uri) {
        this(id, urn, authorityTld, authorityServer, slices, projects, uri,
                JsonLdObjectsMetaData.mustSerializeAsEmbeddedObjectForJacksonConstructor());
    }
    User(String id,
         GeniUrn urn,
         String authorityTld,
         URL authorityServer,
         List<Slice>  slices,
         List<Project> projects,
         URI uri,
         boolean serializeInfo_SerializeAsEmbeddedObject) {
        super(id, uri, serializeInfo_SerializeAsEmbeddedObject);
        this.urn = urn;
        this.authorityTld = authorityTld;
        this.authorityServer = authorityServer;
        this.slices = slices == null ? null : Collections.unmodifiableList(new ArrayList<>(slices));
        this.projects = projects == null ? null : Collections.unmodifiableList(new ArrayList<>(projects));
    }

    @JsonProperty
    @JsonSerialize(using = JsonGeniUrnSerializer.class)
    @JsonDeserialize(using = JsonGeniUrnDeserializer.class)
    public GeniUrn getUrn() {
        return urn;
    }

    @JsonProperty
    public String getAuthorityTld() {
        return authorityTld;
    }

    @JsonProperty
    public URL getAuthorityServer() {
        return authorityServer;
    }

    @JsonProperty
    public List<Slice> getSlices() {
        return slices;
    }

    @JsonProperty
    public List<Project> getProjects() {
        return projects;
    }
}
